/* Program4_4 Uart echo
 *
 * Ez a program egy karaktert fogad az UART2 port bemenetn
 * majd eggyel megnvelve visszarja az UART2 port kimenetre.
 * SystemInit() alaprtelmezetten 10,48576 MHz-re lltja a 
 * buszfrekvencit. A baudrate = bus freq. / BDH:BDL / 16
 * kplet alapjn 9600 bps-t lltunk be. 
 * 
 * Hasznljunk egy USB-soros talaktt a kapcsoldshoz
 * s valamilyen terminl emultor programot (PUTTy.exe, 
 * Termite, TeraTerm stb.) az zenetek megjelentshez!
 * Bellts: 9600 bps, 8N1, adatfolyam vezrls nlkl.
 * A begpelt karakterek eggyel megnvelve lesznek visszatkrzve.
 * Az UART2 Tx kimenett a PTE22, az UART Rx bemenett pedig
 * a PTE23 kivezetsre konfigurljuk.
 *
 * A program forrsa: Mazidi et al., Freescale ARM Cortex-M Embedded Programming
 * http://www.microdigitaled.com/ARM/Freescale_ARM/Code/Ver1/Chapter4/Program4_1.txt
 * Vltoztatsok (Cserny I.): 
 * - Baudrate jraszmolsa az eltro busz orajel frekvencia miatt
 * - Kivezetsek tkonfigurlsa PTE22, PTE23-ra
 * - A visszatkrztt karakter kdjt eggyel megnveljuk
 */

#include <MKL25Z4.h>

void UART2_init(void);                   // UART2 kimenet inicializlsa 

int main (void) {
    char c;
    UART2_init();   	
    while (1) {
        while(!(UART2->S1 & 0x20));      // Karakter berkezsre vrunk
        c = UART2->D ;                   // Beolvassuk a vett karaktert
        while(!(UART2->S1 & 0x80));      // Vrunk mg kirl a kldo buffer
        UART2->D = c+1;                  // Kikldjk az eggyel megnvelt kdot
    }
}

/* UART2 Tx (PTE22) inicializlsa 9600 Baud tviteli sebessgre
 * Baudrate = bus freq / 16 / BDH:BDL
 * Esetnkben 10.49 MHz / 16 / 0x00:0x44 = 9637 bps, 
 * ami elfogadhat eltrs a nvlegestol.
 * Megjegyzs: UART2 rajele a buszfrekvencia, tlmintavtelezs arnya mindig 16
 */
void UART2_init(void) {
    SIM->SCGC4 |= 0x1000;        // UART2 perifria engedlyezse
    UART2->C2 = 0;               // UART2 letiltsa a konfigurls idotartamra
    UART2->BDH = 0x00;           // Baudrate = 9600 bps (bit8 - bit12) 
    UART2->BDL = 0x44;           // Baudrate = 9600 bps (bit0 - bit7)
    UART2->C1 = 0x00;            // 8-bites adatformtum
	  UART2->C3 = 0x00;            // Nincs hibainterrupt
    UART2->C2 = 0x0C;            // Adatklds s fogads engedlyezse

    SIM->SCGC5 |= 0x2000;        // PORTE engedlyezse (bit 13)
    PORTE->PCR[22] = 0x0400;     // PTE22 legyen UART2_Tx zemmdban (Alt4)
	  PORTE->PCR[23] = 0x0400;     // PTE23 legyen UART2_Rx zemmdban (Alt4)
}
