#include <MKL25Z4.H>

// Initialize USART to simple polling mode (no interrupt)
void UART_config(void) {
  uint32_t SBR;
  uint32_t OSR;
	
  /* SIM_SCGC5: PORTA=1 */
  SIM->SCGC5 |= SIM_SCGC5_PORTA_MASK; // Enable clock gate for ports to enable pin routing
	
  SIM->SCGC4 |= SIM_SCGC4_UART0_MASK; // Enable clocks to UART0 module

  /* PORTA_PCR1: ISF=0,MUX=2 */
  PORTA->PCR[1] |= PORT_PCR_MUX(0x02); //set PTA1 to UART0_RX
  /* PORTA_PCR2: ISF=0,MUX=2 */
  PORTA->PCR[2] |= PORT_PCR_MUX(0x02); //set PTA2 to UART0_TX	

  /* Disable TX & RX while we configure settings */
  UART0->C2 &= ~(UART0_C2_TE_MASK); //disable transmitter
  UART0->C2 &= ~(UART0_C2_RE_MASK); //disable receiver
 
  /* UART0_C1: LOOPS=0,DOZEEN=0,RSRC=0,M=0,WAKE=0,ILT=0,PE=0,PT=0 */
  UART0->C1 = 0x00U; /* Set the C1 register */
  /* UART0_C3: R8T9=0,R9T8=0,TXDIR=0,TXINV=0,ORIE=0,NEIE=0,FEIE=0,PEIE=0 */
  UART0->C3 = 0x00U; /* Set the C3 register */
  /* UART0_S2: LBKDIF=0,RXEDGIF=0,MSBF=0,RXINV=0,RWUID=0,BRK13=0,LBKDE=0,RAF=0 */
  UART0->S2 = 0x00U; /* Set the S2 register */
 
  // set clock source to be from PLL
  SIM->SOPT2 |= (SIM_SOPT2_PLLFLLSEL_MASK | SIM_SOPT2_UART0SRC(1));
 
  /*
  * Target Baud rate = 38400
  *
  * sys clock = FLL/PLL = 48.000MHz
  * Baud rate = sys clock / ((OSR+1) * SBR)
  * OSR = 3
  * SBR = 312
  *
  * Resulting Baud rate = 48MHz / ((3 + 1) * 312) = 38461.5
  */
  SBR = 312; //Set the baud rate register, SBR = 312
  UART0->BDH  = (UART0_BDH_SBR_MASK) & (SBR >> 8);
  UART0->BDL  = (UART0_BDL_SBR_MASK) & (SBR & 0xFF);
 
  OSR = 3; // set the oversampling ratio to option #3 = 4x
  UART0->C4 &= (~UART0_C4_OSR_MASK) | OSR;
 
  UART0->C5 |= UART0_C5_BOTHEDGE_MASK; //enable sampling on both edges of the clock 
  UART0->C2 |= UART0_C2_TE_MASK; //enable transmitter
  UART0->C2 |= UART0_C2_RE_MASK; //enable receiver

  return;
}
// Output a character to UART0
char UART_putc(char ch) {
  /* Wait if Transmit Data Register Empty flag is 0 */
  while ((UART0->S1 & UART0_S1_TDRE_MASK) == 0);
  UART0->D = ch; // send a character
  return ch;
}
// Read a character from UART0. If no data received yet, wait
char UART_getc(void) {
	/* Wait if Receive Data Register Full flag is 0 */
  while ((UART0->S1 & UART0_S1_RDRF_MASK) == 0); // 
  return UART0->D;
}
// Echo UART input if there is any
void UART_echo(void) {
  if ((UART0->S1 & UART0_S1_RDRF_MASK)) {
    UART_putc(UART0->D);}
  return;
}

// Uart string output
void UART_puts(char * mytext) {
  char CurrChar;
  CurrChar = *mytext;
  while (CurrChar != (char) 0x0){
    UART_putc(CurrChar);  // Normal data
    mytext++;
    CurrChar = *mytext;
    }
  return;
}

